/*
 * Simple Editor v0.6 - MainWindow Version
 *
 * Demo application for Simple Framework.
 * Based on Qt 4.7 MainWindow C++ Example from the QtSDK.
 *
 * Copyleft 2012, Valentin Schmidt
 * Date: 2012-05-01
 */

this.scriptfile = "main.js"; // useful for debugging

var editor = {
	version:0.6,
	mainWindowID:0,
	menuBarID:0,
	toolBarID:0,
	statusBarID:0,
	editID:0,
	curFile:"",
	isModified:false,
	searchStr:"",
	printerID:0,
	actions:{}
};

//--------------------------------------
// EVENT HANDLER CALLED BY RUNTIME
//--------------------------------------
function startMovie(){
	try{

	// JS DEBUGGING CONSOLE (uncomment to show console)
	//if ($.jsconsole) $.jsconsole.open();

	// SET DEFAULT ALERTHOOK (ONLY USED FOR UNCAUGHT EXCEPTIONS)
	$.setAlertHook();

	// MAKE PLAYHEAD LOOP ON CURRENT FRAME
	$.setFrameLoop();

	// INITIALIZE QT
	$.qtInit(20); // optional value: idle sleep time in ms: higher value=lower app CPU usage

	// INITIALIZE EDITOR APP
	editor.init();

	}catch(e){eval($.CATCH);}
}

//--------------------------------------
// EVENT HANDLER CALLED BY RUNTIME
//--------------------------------------
function stopMovie(){
	try{
	
	// DESTROY
	$.qtCallWidget(editor.mainWindowID, "destroy");
	$.qtDestroy();

	}catch(e){eval($.CATCH);}
}

//--------------------------------------
// EVENT HANDLER CALLED BY RUNTIME
//--------------------------------------
function closeRequest(){
  if (editor.maybeSave()) {
		editor.writeSettings();
		_player.quit();
  }
}

//--------------------------------------
// EVENT HANDLER CALLED BY RUNTIME
//--------------------------------------
function activateApplication(){
  $.qtSetIdleSleep(20); // idle sleep time in ms: higher value=lower app CPU usage
}

//--------------------------------------
// EVENT HANDLER CALLED BY RUNTIME
//--------------------------------------
function deactivateApplication(){
	$.qtSetIdleSleep(100); // idle sleep time in ms: higher value=lower app CPU usage
}

/**
 * editor.init
 * Initializes the editor app
 */
editor.init = function(){

  // CREATE MAINWINDOW
  editor.mainWindowID = $.qtNewWindow("MainWindow", 0); // NO PARENT, THEREFOR MUST BE MANUALLY DESTROYED

  // CREATE PLAINTEXTEDIT
  editor.editID = $.qtNewWidget("PlainTextEdit", editor.mainWindowID);
	$.qtCallWidget(editor.mainWindowID, "setCentralWidget", editor.editID);

  $.qtSetWidgetCallback(editor.editID, "textChanged", function(widgetID){
  	editor.isModified = true;
  	var shownName = editor.curFile;
  	if (!editor.curFile) shownName = "untitled.txt";
  	$.qtCallWidget(editor.mainWindowID, "setWindowTitle", "Simple Editor - " + shownName + "*");
  });

	var settings = 	editor.readSettings();
	if (!settings){
		var screen = _system.deskTopRectList[1];
		settings = {
			toolbar:1,
			statusbar:1,
			state:0,
			x:screen.width/2-400,
			y:screen.height/2-300,
			w:800,
			h:600
		}
	}
	
	editor.createActions(settings);
  editor.createMenuBar();
  editor.createToolBar(settings);
	editor.createStatusBar(settings);
	editor.setCurrentFile("");

	$.qtCallWidget(editor.mainWindowID, "resize", settings.w, settings.h);
	$.qtCallWidget(editor.mainWindowID, "move", settings.x, settings.y);
	$.qtCallWidget(editor.mainWindowID, "setWindowState", settings.state);
	
	// DEFINE DROP CALLBACK
  $.qtSetDropCallback(function(dropObj){
		try{
	  	if (editor.maybeSave()){
				var fn = dropObj['application/x-qt-windows-mime;value="FileName"'];
				editor.loadFile(fn);
			}
		}catch(e){eval($.CATCH);}
  });

  $.qtCallWidget(editor.mainWindowID, "setWindowIcon", _movie.path+"assets"+$.PD+"images"+$.PD+"icon.png");

  // QUIT APPLICATION WHEN MAIN WINDOW WAS CLOSED
  $.qtSetEventCallback(editor.mainWindowID, 19, closeRequest);
  
	// SHOW
	$.qtCallWidget(editor.mainWindowID, "show");
}

/**
 * editor.createActions
 * Creates actions used by MenuBar and ToolBar
 */
editor.createActions = function(settings){

	// FILE 

  editor.actions.newAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.newAct, "setText", "&New");
  $.qtCallObject(editor.actions.newAct, "setIcon", _movie.path+"assets"+$.PD+"images"+$.PD+"new.png");
  $.qtCallObject(editor.actions.newAct, "setShortcut", "Ctrl+N");
  $.qtCallObject(editor.actions.newAct, "setStatusTip", "Create a new file");
  $.qtSetObjectCallback(editor.actions.newAct, "triggered", editor.newFile);

  editor.actions.openAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.openAct, "setText", "&Open...");
  $.qtCallObject(editor.actions.openAct, "setIcon", _movie.path+"assets"+$.PD+"images"+$.PD+"open.png");
  $.qtCallObject(editor.actions.openAct, "setShortcut", "Ctrl+O");
  $.qtCallObject(editor.actions.openAct, "setStatusTip", "Open an existing file");
  $.qtSetObjectCallback(editor.actions.openAct, "triggered", editor.open);

  editor.actions.saveAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.saveAct, "setText", "&Save");
  $.qtCallObject(editor.actions.saveAct, "setIcon", _movie.path+"assets"+$.PD+"images"+$.PD+"save.png");
  $.qtCallObject(editor.actions.saveAct, "setShortcut", "Ctrl+S");
  $.qtCallObject(editor.actions.saveAct, "setStatusTip", "Save the document to disk");
  $.qtSetObjectCallback(editor.actions.saveAct, "triggered", editor.save);

  editor.actions.saveAsAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.saveAsAct, "setText", "Save &As...");
  $.qtCallObject(editor.actions.saveAsAct, "setStatusTip", "Save the document under a new name");
  $.qtSetObjectCallback(editor.actions.saveAsAct, "triggered", editor.saveAs);

  editor.actions.pageSetupAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.pageSetupAct, "setText", "Page Setup");
  //$.qtCallObject(editor.actions.pageSetupAct, "setStatusTip", "Page Setup");
  $.qtSetObjectCallback(editor.actions.pageSetupAct, "triggered", function(){
  	if (!editor.printerID) editor.printerID = $.qtNewObject("Printer");
  	if (editor.printerID){
			$.qtNewDialog("PageSetupDialog", editor.mainWindowID, editor.printerID);
		}
  });

  editor.actions.printAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.printAct, "setText", "Print");
  $.qtCallObject(editor.actions.printAct, "setStatusTip", "Print current text");
  $.qtSetObjectCallback(editor.actions.printAct, "triggered", function(){
  	if (!editor.printerID) editor.printerID = $.qtNewObject("Printer");
  	if (editor.printerID){
	  	var ok = $.qtNewDialog("PrintDialog", editor.mainWindowID, editor.printerID);
	  	if (ok) $.qtCallWidget(editor.editID, "print", editor.printerID);
	  }
  });

  editor.actions.exitAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.exitAct, "setText", "E&xit");
  $.qtCallObject(editor.actions.exitAct, "setShortcut", "Ctrl+Q");
  $.qtCallObject(editor.actions.exitAct, "setStatusTip", "Exit the application");
  $.qtSetObjectCallback(editor.actions.exitAct, "triggered", closeRequest);

  // EDIT

  editor.actions.undoAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.undoAct, "setText", "&Undo");
  $.qtCallObject(editor.actions.undoAct, "setShortcut", "Ctrl+Z");
  //$.qtCallObject(editor.actions.undoAct, "setStatusTip", "Undo");
  $.qtSetObjectCallback(editor.actions.undoAct, "triggered", function(){
  	$.qtCallWidget(editor.editID, "undo");
  });

  editor.actions.cutAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.cutAct, "setText", "Cu&t");
  $.qtCallObject(editor.actions.cutAct, "setIcon", _movie.path+"assets"+$.PD+"images"+$.PD+"cut.png");
  $.qtCallObject(editor.actions.cutAct, "setShortcut", "Ctrl+X");
  $.qtCallObject(editor.actions.cutAct, "setStatusTip", "Cut the current selection's contents to the clipboard");
  $.qtSetObjectCallback(editor.actions.cutAct, "triggered", function(){
  	$.qtCallWidget(editor.editID, "cut");
  });

  editor.actions.copyAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.copyAct, "setText", "&Copy");
  $.qtCallObject(editor.actions.copyAct, "setIcon", _movie.path+"assets"+$.PD+"images"+$.PD+"copy.png");
  $.qtCallObject(editor.actions.copyAct, "setShortcut", "Ctrl+C");
  $.qtCallObject(editor.actions.copyAct, "setStatusTip", "Copy the current selection's contents to the clipboard");
  $.qtSetObjectCallback(editor.actions.copyAct, "triggered", function(){
  	$.qtCallWidget(editor.editID, "copy");
  });

  editor.actions.pasteAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.pasteAct, "setText", "&Paste");
  $.qtCallObject(editor.actions.pasteAct, "setIcon", _movie.path+"assets"+$.PD+"images"+$.PD+"paste.png");
  $.qtCallObject(editor.actions.pasteAct, "setShortcut", "Ctrl+V");
  $.qtCallObject(editor.actions.pasteAct, "setStatusTip", "Paste the clipboard's contents into the current selection");
  $.qtSetObjectCallback(editor.actions.pasteAct, "triggered", function(){
  	$.qtCallWidget(editor.editID, "paste");
  });

  editor.actions.clearAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.clearAct, "setText", "Clear");
  //$.qtCallObject(editor.actions.clearAct, "setShortcut", "Del");
  //$.qtCallObject(editor.actions.clearAct, "setStatusTip", "Clear");
  $.qtSetObjectCallback(editor.actions.clearAct, "triggered", function(){
  	$.qtCallWidget(editor.editID, "clear");
  });


  editor.actions.findAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.findAct, "setText", "&Find");
  $.qtCallObject(editor.actions.findAct, "setShortcut", "Ctrl+F");
  //$.qtCallObject(editor.actions.findAct, "setStatusTip", "Find");
  $.qtSetObjectCallback(editor.actions.findAct, "triggered", function(){
  	editor.searchStr = $.qtNewDialog(
			"InputDialog",
			editor.mainWindowID, 
			"Find String", //title
			"Search for:", //label
			"" //text
		);
		if (editor.searchStr) $.qtCallWidget(editor.editID, "findText", editor.searchStr);
  });

  editor.actions.findAgainAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.findAgainAct, "setText", "Find again");
  //$.qtCallObject(editor.actions.findAgainAct, "setShortcut", "F3");
  //$.qtCallObject(editor.actions.findAgainAct, "setStatusTip", "Find ");
  $.qtSetObjectCallback(editor.actions.findAgainAct, "triggered", function(){
		if (editor.searchStr) $.qtCallWidget(editor.editID, "findText", editor.searchStr);
  });

	// VIEW

  editor.actions.toolBarAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.toolBarAct, "setText", "Show ToolBar");
  $.qtCallObject(editor.actions.toolBarAct, "setCheckable", 1);
  $.qtCallObject(editor.actions.toolBarAct, "setChecked", settings.toolbar);  	
  $.qtSetObjectCallback(editor.actions.toolBarAct, "toggled", function(actionID, checked){
    $.qtCallWidget(editor.toolBarID, "setVisible", checked);
  });
  
  editor.actions.statusBarAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.statusBarAct, "setText", "Show StatusBar");
  $.qtCallObject(editor.actions.statusBarAct, "setCheckable", 1);
  $.qtCallObject(editor.actions.statusBarAct, "setChecked", settings.statusbar);
  $.qtSetObjectCallback(editor.actions.statusBarAct, "toggled", function(actionID, checked){
    $.qtCallWidget(editor.statusBarID, "setVisible", checked);
  });
  
	// HELP

  editor.actions.showConsoleAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.showConsoleAct, "setText", "Show JS Console");
  $.qtSetObjectCallback(editor.actions.showConsoleAct, "triggered", function(){
		if ($.jsconsole) $.jsconsole.open();
  });
  
  editor.actions.aboutAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.aboutAct, "setText", "&About");
  $.qtCallObject(editor.actions.aboutAct, "setStatusTip", "Show the application's About box");
  $.qtSetObjectCallback(editor.actions.aboutAct, "triggered", function(){  	
		$.qtNewDialog(
			"MessageBox",
			editor.mainWindowID, 
			"About",
			"<b>Simple Editor v"+editor.version+"</b><br><br>The Simple Editor example demonstrates how to write modern GUI applications with a menu bar, toolbars, and a status bar using Simple Framework and Qt.",
			"about"
		);
		
  });

  editor.actions.aboutQtAct = $.qtNewObject("Action");
  $.qtCallObject(editor.actions.aboutQtAct, "setText", "About &Qt");
	$.qtCallObject(editor.actions.aboutQtAct, "setStatusTip", "Show the Qt library's About box");
	$.qtSetObjectCallback(editor.actions.aboutQtAct, "triggered", function(){
		$.qtNewDialog(
			"MessageBox",
			editor.mainWindowID, 
			"About Qt",
			"",
			"aboutQt"
		);
  });
}

/**
 * editor.createMenuBar
 * Creates a MenuBar.
 */
editor.createMenuBar = function(){

  editor.menuBarID = $.qtNewWidget("MenuBar", editor.mainWindowID);
  $.qtCallWidget(editor.mainWindowID, "setMenuBar", editor.menuBarID);

	var fileMenu = $.qtCallWidget(editor.menuBarID, "addMenu", "&File");
	$.qtCallWidget(fileMenu, "addAction", editor.actions.newAct);
	$.qtCallWidget(fileMenu, "addAction", editor.actions.openAct);
	$.qtCallWidget(fileMenu, "addAction", editor.actions.saveAct);
	$.qtCallWidget(fileMenu, "addAction", editor.actions.saveAsAct);
	$.qtCallWidget(fileMenu, "addSeparator");
	$.qtCallWidget(fileMenu, "addAction", editor.actions.pageSetupAct);
	$.qtCallWidget(fileMenu, "addAction", editor.actions.printAct);
	$.qtCallWidget(fileMenu, "addSeparator");
	$.qtCallWidget(fileMenu, "addAction", editor.actions.exitAct);

	var editMenu = $.qtCallWidget(editor.menuBarID, "addMenu", "&Edit");
	$.qtCallWidget(editMenu, "addAction", editor.actions.undoAct);
	$.qtCallWidget(editMenu, "addSeparator");
	$.qtCallWidget(editMenu, "addAction", editor.actions.cutAct);
	$.qtCallWidget(editMenu, "addAction", editor.actions.copyAct);
	$.qtCallWidget(editMenu, "addAction", editor.actions.pasteAct);
	$.qtCallWidget(editMenu, "addAction", editor.actions.clearAct);
	$.qtCallWidget(editMenu, "addSeparator");
	$.qtCallWidget(editMenu, "addAction", editor.actions.findAct);
	$.qtCallWidget(editMenu, "addAction", editor.actions.findAgainAct);

	var viewMenu = $.qtCallWidget(editor.menuBarID, "addMenu", "&View");
	$.qtCallWidget(viewMenu, "addAction", editor.actions.toolBarAct);
	$.qtCallWidget(viewMenu, "addAction", editor.actions.statusBarAct);
	
	var helpMenu = $.qtCallWidget(editor.menuBarID, "addMenu", "&Help");
	$.qtCallWidget(helpMenu, "addAction", editor.actions.showConsoleAct);
	$.qtCallWidget(helpMenu, "addSeparator");
	$.qtCallWidget(helpMenu, "addAction", editor.actions.aboutAct);
	$.qtCallWidget(helpMenu, "addAction", editor.actions.aboutQtAct);
}

/**
 * editor.createToolBar
 * Creates a ToolBar.
 */
editor.createToolBar = function(settings){
  editor.toolBarID = $.qtNewWidget("ToolBar", editor.mainWindowID, "ToolBar");
  $.qtCallWidget(editor.mainWindowID, "addToolBar", editor.toolBarID);
  $.qtSetWidgetCallback(editor.toolBarID, "visibilityChanged", function(widgetID, vis){
		// update menu
		$.qtCallObject(editor.actions.toolBarAct, "setChecked", vis);
  });
  $.qtCallWidget(editor.toolBarID, "addAction", editor.actions.newAct);
	$.qtCallWidget(editor.toolBarID, "addAction", editor.actions.openAct);
	$.qtCallWidget(editor.toolBarID, "addAction", editor.actions.saveAct);
	$.qtCallWidget(editor.toolBarID, "addSeparator");
  $.qtCallWidget(editor.toolBarID, "addAction", editor.actions.cutAct);
	$.qtCallWidget(editor.toolBarID, "addAction", editor.actions.copyAct);
	$.qtCallWidget(editor.toolBarID, "addAction", editor.actions.pasteAct);
	$.qtCallWidget(editor.toolBarID, "setVisible", settings.toolbar);
}

/**
 * editor.createStatusBar
 * Creates a StatusBar.
 */
editor.createStatusBar = function(settings){
  editor.statusBarID = $.qtNewWidget("StatusBar", editor.mainWindowID);
  $.qtCallWidget(editor.mainWindowID, "setStatusBar", editor.statusBarID);
  $.qtCallWidget(editor.statusBarID, "setSizeGripEnabled", false);
  $.qtCallWidget(editor.statusBarID, "showMessage", "Ready.");
	$.qtCallWidget(editor.statusBarID, "setVisible", settings.statusbar);
}

/**
 * editor.readSettings
 * Reads settings from registry, returns object (or false).
 */
editor.readSettings = function(){
	var settingsID = $.qtNewObject("Settings", 0, "Valentin Schmidt", "Simple Editor");
	var str = $.qtCallObject(settingsID, "value", "settings");
	return str ? $.parseJSON(str) : false;
}

/**
 * editor.writeSettings
 * Writes settings to registry (as JSON string).
 */
editor.writeSettings = function(){
	var p, settings = {};
	settings.state = $.qtCallWidget(editor.mainWindowID, "windowState");
	if (settings.state==2){//fullscreen		
		var screen = _system.deskTopRectList[1];
		settings.x = screen.width/2-400;
		settings.y = screen.height/2-300;
		settings.w = 800;
		settings.h = 600;
	}else{
		p = $.qtCallWidget(editor.mainWindowID, "pos");
		settings.x = p.locH;
		settings.y = p.locV;
		p = $.qtCallWidget(editor.mainWindowID, "size");
		settings.w = p.locH;
		settings.h = p.locV;
	}
	settings.toolbar = $.qtCallWidget(editor.toolBarID, "isVisible");
	settings.statusbar = $.qtCallWidget(editor.statusBarID, "isVisible");
	var settingsID = $.qtNewObject("Settings", 0, "Valentin Schmidt", "Simple Editor");
	$.qtCallObject(settingsID, "setValue", "settings", $.toJSON(settings));
}

/**
 * editor.newFile
 * Handles New action.
 */
editor.newFile = function(){
	try{
	  if (editor.maybeSave()) {
	    $.qtCallWidget(editor.editID, "clear");
	   	editor.setCurrentFile("");
	  }
	}catch(e){eval($.CATCH);}
}

/**
 * editor.open
 * Handles Open action.
 */
editor.open = function(){
	try{
	  if (editor.maybeSave()){
	  	var fn = $.qtNewDialog("FileOpenDialog", editor.mainWindowID, "", "", "");
			if (fn) editor.loadFile(fn);
		}
	}catch(e){eval($.CATCH);}
}

/**
 * editor.save
 * Handles Save action.
 */
editor.save = function(){
	try{
	  if (!editor.curFile){
			return editor.saveAs();
	  } else {
			return editor.saveFile(editor.curFile);
	  }
	}catch(e){eval($.CATCH);}
}

/**
 * editor.saveAs
 * Handles SaveAs action.
 */
editor.saveAs = function(){
	try{
	  var fn = $.qtNewDialog("FileSaveAsDialog", editor.mainWindowID, "", editor.curFile, "");
	  if (!fn) return false;
		return editor.saveFile(fn);
	}catch(e){eval($.CATCH);}
}

/**
 * editor.maybeSave
 */
editor.maybeSave = function(){
  if (editor.isModified){
		var ret = $.qtNewDialog(
			"MessageBox",
			editor.mainWindowID,
			"Editor",
			"The document has been modified.\nDo you want to save your changes?",
			"warning",
			0x00000800 | 0x00800000 | 0x00400000 // QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel
		);
    if (ret == 0x00000800)// QMessageBox::Save
			return editor.save();
    else if (ret == 0x00400000) //QMessageBox::Cancel
			return false;
  }
  return true;
}

/**
 * editor.loadFile
 * Loads given file.
 *
 * @param fn {String}
 */
editor.loadFile = function(fn){
  var str = $.file_get_contents(fn);
  if (str===false){
		$.qtNewDialog(
			"MessageBox",
			editor.mainWindowID,
			"Editor",
			"Cannot read file "+fn,
			"warning"
		);
		return;
	}
  $.qtCallWidget(editor.editID, "setPlainText", str);
	editor.setCurrentFile(fn);
  $.qtCallWidget(editor.statusBarID, "showMessage", "File loaded.", 2000);
}

/**
 * editor.saveFile
 * Saves given file.
 *
 * @param fn {String}
 */
editor.saveFile = function(fn){
  var str = $.qtCallWidget(editor.editID, "toPlainText");
  var ok = $.file_put_contents(fn, str);
  if (!ok){
		$.qtNewDialog(
			"MessageBox",
			editor.mainWindowID,
			"Editor",
			"Cannot write file "+fn,
			"warning"
		);
		return false;
	}
  editor.setCurrentFile(fn);
  $.qtCallWidget(editor.statusBarID, "showMessage", "File saved.", 2000);
  return true;
}

/**
 * editor.setCurrentFile
 * Sets current file to the given file.
 *
 * @param fn {String}
 */
editor.setCurrentFile = function(fn){
  editor.curFile = fn;
	editor.isModified = false;
  var shownName = editor.curFile?editor.curFile:"untitled.txt";
  $.qtCallWidget(editor.mainWindowID, "setWindowTitle", "Simple Editor - " + shownName);
}
